--[[ Displays one-liners
	arguments:
		pattern		--search for a certain one-liner, can include * which matches any text
		number		--get a one-liner by number
	
	examples:
		1liners		--random one-liner
		1liners 32	--one-liner number 32
		1liners All*	--search for one-liners that start with the word 'All'
--]]

_G.PowerChat = _G.PowerChat or {}
_G.PowerChatCommands = _G.PowerChatCommands or {}

local cmdName = '1liners'

local chatName = '1LINERS'

PowerChatCommands[cmdName] = {}
PowerChatCommands[cmdName].Parse = function(args)
	table.remove(args, 1)
	
	local result = ''

	if #args > 0 then
		for i,s in pairs(args) do
			local n = tonumber(s)
			
			if n then
				if n < 1 or n > #(PowerChatCommands[cmdName].Texts) then
					PowerChat:OutPut(chatName, 'The index must be from 1 to '..#(PowerChatCommands[cmdName].Texts), Color.green)
				else
					PowerChat:OutPut(chatName, '#'..n..': '..PowerChatCommands[cmdName].Texts[n], Color.green)
				end
			else
				local slow = ('^'..s:lower()..'$'):gsub('%*', '.*')

				for k,v in pairs(PowerChatCommands[cmdName].Texts) do
					if v:lower():match(slow) then
						PowerChat:OutPut(chatName, '#'..k..': '..v, Color.green)
					end
				end
			end
		end
	else
		local num = math.random(#PowerChatCommands[cmdName].Texts)
		PowerChat:OutPut(chatName, '#'..num..': '..PowerChatCommands[cmdName].Texts[num], Color.green)
	end
end

PowerChatCommands[cmdName].Help = function()
	PowerChat:OutPut(chatName, 'Displays one-liners', Color.yellow)
	PowerChat:OutPut(chatName, '1liners [pattern] [number]', Color.yellow)
end

PowerChatCommands[cmdName].Texts = 
		{
"\"Automatic\" simply means that you cannot repair it yourself.","90% of everything is crud.","A Smith & Wesson always beats four aces.","A bird in hand is safer than one overhead.","A clean desk is a sign of a sick mind.","A closed mouth says nothing wrong; a closed mind does nothing right.","A dog is a dog except when he is facing you. Then he is Mr. Dog.","A fool and his money are some party.","A friend in power is a friend lost.","A great deal of money is never enough once you have it.","A hunch is creativity trying to tell you something.","A man will believe anything that does not cost him anything.","A perfectly calm day will turn gusty the instant you drop a $20 bill.",
"A sense of decency is often a decent man's undoing.","A short cut is the longest distance between two points.","A stockbroker is someone who invests your money until it is all gone.","A synonym is a word you use when you can't spell the other one.","A waist is a terrible thing to mind.","Academic rivalries are so intense because the stakes are so small.","Activity is the politician's substitute for achievement.","After all is said and done, a hell of a lot more is said than done.","Against stupidity, the gods themselves contend in vain.","All great discoveries are made by mistake.",
"All skill is in vain when an angel pees in the barrel of your rifle.","All things are possible. Except skiing through a revolving door.","All turtle thoughts are of turtle.","All warranties expire upon payment of invoice.","All work and no play make Jack a dull boy and Jill a wealthy widow.","All's well that ends.","Almost everything in life is easier to get into than out of.","Always forgive your enemies - nothing else annoys them as much.","Always mistrust a subordinate who never finds fault with his boss.","Ambition is the curse of the political class.",
"An efficient bureaucracy is the greatest threat to liberty.","An elephant is a mouse built to government specifications.","An infinite number of mediocrities do not add up to one genius.","An open mouth oft-times accompanies a closed mind.","An unhappy crew makes for a dangerous voyage.","Anger is never without an argument, or with a good one.","Any given computer program, when running, is obsolete.","Any simple theory will be worded in the most complicated terms.","Any sufficiently advanced technology is indistinguishable from magic.","Anyone can handle a crisis. It's everyday living that kills you.","As the rabbit said, if that ain't a wolf, it's a hell of a big dog.","Ask your children what they want for dinner only if they are buying.","Bad weather forecasts are more often right than good ones.","Bankers are the assassins of hope.","Basic research is what you do when you don't know what you are doing.",
"Be a corporate good citizen; hire the morally handicapped.","Beauty is only skin deep, but ugly goes right to the bone.","Being king is not much fun if no one knows you are one.","Bend the facts to fit the conclusion. It's easier that way.","Beware of all enterprises requiring new clothes.","Black holes are outa sight!","Bosses come and bosses go, but a good secretary lasts forever.","By the time most of us have money to burn, our fire's gone out.","The first hundred years are the hardest!","Civil servants are neither civil nor servile.","Common sense is instinct. Enough of it is genius.","Confidence is the feeling you had before you knew better.","Crime is merely politics without the excuses.","Crisis management works beautifully until an actual crisis occurs.",
"Da trouble wit computers is, dey got no sense of humor.","Desperate diseases require desperate remedies.","Distrust your first impressions; they are invariably too favorable.","Don't be so humble...you aren't that great.","Don't get married if you are afraid of solitude.","Don't wear earmuffs in a bed of rattlesnakes.","Enthusiasm wanes, but dullness lasts forever.","Eternity is a terrible thought...where will it all end","Even the most faithful believer can serve a false god.","Every calling is great when greatly pursued.","Every family tree has some sap.","Everybody's death simplifies life for someone.","Everyone has a scheme for getting rich that will not work.","Excellence is an option that is renewable.","Expectations should not determine whether or not one acts, nor how.","Experience is a good teacher, but submits huge bills.",
"Experience is what you get when you don't get what you wanted.","Experimentation is the mother of confusion.","Extinction is the ultimate fate of all species.","Extreme boredom serves to cure boredom.","Extreme sorrow laughs; extreme joy weeps.","Facts cannot prevail against faith, or adamant folly.","Failure is a measurement that depends on the standard applied.","Fear is no great respecter of reason.","Feed the wolf as you will; he will always look to the forest.","Fill what's empty. Empty what's full. And scratch where it itches.",
"For a man of fortitude, there are no walls, only avenues.","For peace of mind, resign as general manager of the universe.","Freedom is for everyone. Or no one.","Freedom of the press is limited to those who have one.","Friends may come and friends may go, but enemies accumulate.","Frustration is not having anyone else to blame but one's self.","Get too many irons in your fire and you'll put it out.","God can't alter history, so he created historians.","God has Alzheimer's disease; he's forgotten that we exist.","God made everything out of nothing. But the nothingness shows through.","Government corruption seems always to be reported in the past tense.","Have a nice day...somewhere else.","He who lives on hope has a slender diet indeed.","He who looks too far ahead stumbles over his own boots.",
"He who would climb to the top must leave much behind.","He who would leap high must take a long run.","He who would pursue revenge should first dig two graves.","Hell is not a place. Hell is what hurts worst.","History is the sum total of things that could have been avoided.","Honesty in politics is much like oxygen.","The higher up you go, the scarcer it becomes.","How can you tell when a salesman is lying? When his lips are moving.","If all else fails, read the destructions.","If an experiment works, something has gone wrong.","If it jams - force it. If it breaks, it needed replacing anyway.","If the shoe fits, you're not allowing for growth.","If the thought of growing old bothers you, consider the alternative.","If the universe is indeed insane, who is the asylum keeper?",
"If you are feeling good, don't worry; you'll get over it.","If you cannot convince them, confuse them.","If you cannot understand it, it is intuitively obvious.","If you disinfect the pond, you kill the lilies.","If you live among the wolves, learn to howl like them.","If you try to please everybody, nobody will like it.","In a mad world, only greater madness succeeds.","In case of doubt, make it sound convincing.","In death, avoid hell. In life, avoid the law courts.","In doing good, avoid notoriety. In doing evil, avoid self-awareness.","In hell, treason is the work of angels.",
"In the long run, we are all dead.","In third-world politics, the people with the guns call the shots.","Injustice anywhere is a threat to justice everywhere.","Inside every short man is a tall man doubled over in extreme pain.","Interesting history is awful living.","It is better to be envied than to be consoled.","It is morally wrong to allow suckers to keep their money.","It is often easier to find the truth than it is to accept it.","It is the manner, and not the content, that marks a gentleman.","It is wrong to repeat gossip, but what else can you do with it ?","It's difficult to soar with the eagles when you work with turkeys.","It's not whether you win or lose, it's how you place the blame.","Justice must not only be done; it must be seen to be believed.","Knowledge can cure ignorance, but intelligence cannot cure stupidity.","Labor disgraces no man, but often a man disgraces labor.","Large brains can contain small minds.",
"Law remains long after justice flees.","Less of a good thing is sometimes better - ask anyone on a diet.","Life is a learning experience; the diploma is your death certificate.","Life is a temporary assignment.","Life is a terminal condition.","Life's a bitch.","Live every day as though it were your last.","One day, you'll be right.","Live within your income, even if you must borrow to do it.","Locks and keys are for honest people.","Lost causes are the only ones worth fighting for.","Luck, it is said, dislikes working double shifts.","Many know how to flatter; few know how to praise.","Marriage is like burning the house down to toast the bread.",
"Marriage is the only adventure open to the cowardly.","May you live in interesting times.","Measure twice 'cause you can only cut once.","Middle age is when you wonder if your warranty is running out.","Miles aren't the only distance.","More men are sheep in wolves' clothing than the other way around.","Most men and nations die lying down.","Mother Nature applies all her rules...all the time.","Nature always sides with the hidden flaw.","Never argue with a fool...people may not be able to tell you apart.","Never claim as a right that which you can ask as a favor.","Never climb a fence when you can sit on it.","Never complain; never explain.","Never do card tricks for the group you play poker with.",
"Never embezzle more than your employer can afford.","Never get into a fight with an ugly person. He has nothing to lose.","Never have so many people understood so little about so much.","Never invest in anything that eats or needs repainting.","Never let your sense of morality stop you from doing what is right.","Never marry a woman who prays too much.","Never mistake good manners for good will.","Never play leapfrog with a unicorn.","Never question your wife's judgement...look whom she married.","Never trust anyone who laughs at his own one-liners.","Never underestimate the power of stupidity.","Never ever trust anyone under 30 or over 25.",
"New systems generate new problems.","No man's knowledge goes beyond his experience.","No matter how bad your kid is, he's still good for a tax exemption.","No one can make you feel inferior without your consent.","No one ever found marvels by seeking them.","No one gets too old to learn a new way of being stupid.","Nothing dispels enthusiasm like a small admission fee.","Nothing is illegal if 100 businessmen decide to do it.","Odds are, the phrase \"It's none of my business\" will be followed by \"but\".","Old men make wars. Young men fight them.","Once upon a time, there were two Chinamen. Now look how many there are.","One goddamned thing leads to another goddamned thing.","One sees more clearly backward than forward.",
"Opportunity always knocks at the least opportune moment.","People are always available for work in the past tense.","People who live in a golden age complain that everything looks yellow.","Pessimists are the world's happiest people....","Politicians are much like ships: noisiest when lost in a fog.","Psychopaths aren't born. They are made.","Rabbits dance at the funeral of the lion.",
"Rumors are the sauce of a dry life.","Saints engage in introspection while burly sinners run the world.","Simple pleasures are the last refuge of the complex.","Small things entertain small minds.","Smooth seas never made a good sailor.","Some people can look so busy that they seem indispensable.","Some people can stay longer in an hour than others can in a week.","Songs unheard are sweeter far.","Strong words connote weak arguments.","Succeeding is more satisfying than success.","Success consists of reaching 40 before your waist does.","Support your local bloodhound. Get lost.","Taking something with a grain of salt may raise your blood pressure.","Tatoos are the common man's way of investing in art.",
"Television is chewing gum for the eyes.","Tempt not a desperate man.","The Russians will never invade us...there's no place to park.","The best bilge pump in the world is a terrified sailor with a bucket.","The best way to achieve immortality is by not dying.","The best way to attract money is to give the appearance of having it.","The best way to avoid growing old is not to be born so soon.","The best way to save face is to keep the lower half closed.",
"The biggest idiot can ask questions the smartest man cannot answer.","The deepest and most important virtues are often the dullest ones.","The first great gift that we can bestow upon others is a good example.","The gap between theory and practice is filled with apology.","The man who is always talking about being a gentleman never is one.","The most successful journey is a dull journey.","The mouse dreams dreams that would terrify the cat.","The only imperfect thing in nature is the human race.","The only people to profit from the mistakes of others are biographers.","The only perfect science is hindsight.","The only thing worse than a male chauvinist pig is the female version.","The opera ain't over 'til the fat lady sings.","The person who marries for money generally ends up earning it.","The person who snores the loudest will fall asleep first.","The rat race is over. The rats won.","The real outrage today isn't what's illegal. It's what is legal.","The reason the way of the sinner is hard is because it is so crowded.",
"The repairman will never have seen a model quite like yours before.","The saddest of words: I always wanted to but never did.","The ship of state is the only ship that leaks at the top.","The smaller the issue, the bigger the fight.","The sooner man begins to spend his wealth, the better he uses it.","The strongest part of any paper form is the perforation.","The successful enjoyment of vice requires training and long practice.","The supreme irony of life is that hardly anyone gets out of it alive.","There are no moral messages in Nature.","There are only four basic plots in life, and nine in literature.","There is a time for everything. Mostly, the wrong time.","There is always one more son of a bitch than you counted on.",
"There is no truth in the rumor that man is immortal.","There is nothing like a grievance to sharpen an old man's wits.","There is nothing more terrible than ignorance in action.","Things get worse under pressure.","Those who can - do. Those who can't - teach.","Those totally devoid of useful ability become government economists.","Those who can't teach - administer. Those who can't administer - run for public office.",
"Those who do not follow are dragged.","Those who do not learn from history often end up making it.","Time is a great teacher, but it kills all its pupils.","To a little fish, the waters are always deep.","To a man with a hammer, everything looks like a nail.","To a weary horse, even his own tail is a burden.","To have honesty coupled to beauty is to have honey the sauce to sugar.","To hear tell a hundred times is not as good as once seeing.","To understand the clay is not to understand the pot.","Today's conservative is yesterday's liberal who got mugged lastnight.",
"Too much of anything is wonderful.","Tyranny is always better organized than freedom is.","Virtue is a social liability.","We ain't cheap, but by gosh, we're good!","We don't know one millionth of one per cent about anything.","What people say behind your back is your standing in the community."
		}